require 'ostruct'

module DatabaseHelpers
  def self.init_database(debug = false)
    options = {}
    if debug
      logger = Logger.new(STDOUT)
      logger.level = Logger::DEBUG
      options = { logger: [logger] }
    end
    
    begin
      # if not ocnnected, raises an error
      # this sets the connection once
      Sequel::Model.db
    rescue
      Sequel::Model.db = Sequel.connect('sqlite::memory:', options)
    end

    if Sequel::Model.db.table_exists?(:authenticated_identities)
      Sequel::Model.db.execute_dui "DELETE FROM authenticated_identities;"
    end
    if Sequel::Model.db.table_exists?(:client_addresses)
      Sequel::Model.db.execute_dui "DELETE FROM client_addresses;"
    end
    if Sequel::Model.db.table_exists?(:network_addresses)
      Sequel::Model.db.execute_dui "DELETE FROM network_addresses;"
    end
    if Sequel::Model.db.table_exists?(:policies)
      Sequel::Model.db.execute_dui "DELETE FROM policies;"
    end
    if Sequel::Model.db.table_exists?(:trackers)
      Sequel::Model.db.execute_dui "DELETE FROM trackers;"
    end
  
    config = OpenStruct.new(
      trackers: {},
      policies: [
        {
          "name"=>"default",
          "h_max_addresses"=>5,
          "h_activity_threshold"=>14400,
          "h_interval"=>1800,
          "h_interval_count"=>50,
          "h_grace_threshold"=>1296000,
          "description"=>"Default Policy",
          "interval"=>3600,
          "heuristics"=>false,
          "count"=>10,
          "recipients"=>20,
          "volume"=>"nil"
        },
        {
          "name"=>"default_for_addresses",
          "h_max_addresses"=>5,
          "h_activity_threshold"=>14400,
          "h_interval"=>1800,
          "h_interval_count"=>50,
          "h_grace_threshold"=>1296000,
          "description"=>"Default Policy for unauthenticated addresses",
          "interval"=>86400,
          "heuristics"=>true,
          "count"=>200,
          "recipients"=>200,
          "volume"=>"nil"
        },
        {
          "name"=>"default_for_identities",
          "h_max_addresses"=>5,
          "h_activity_threshold"=>14400,
          "h_interval"=>1800,
          "h_interval_count"=>50,
          "h_grace_threshold"=>1296000,
          "description"=>"Default Policy for authenticated identities",
          "interval"=>86400,
          "heuristics"=>true,
          "count"=>500,
          "recipients"=>1000,
          "volume"=>"nil"
        }
      ]
    )
    
    Postfix::Rate::Models.init(config)
  end
end