require 'postfix/rate/policyd/agent_backend'

request = %Q(request=smtpd_access_policy
protocol_state=RCPT
protocol_name=SMTP
helo_name=some.domain.tld
queue_id=8045F2AB23
sender=foo@bar.tld
recipient=bar@foo.tld
recipient_count=5
client_address=1.2.3.4
client_name=another.domain.tld
reverse_client_name=another.domain.tld
instance=123.456.7
sasl_method=plain
sasl_username=%s
size=0

)

RSpec.describe Postfix::Rate::Policyd::PostfixServer do
  before :each do
    DatabaseHelpers.init_database
  end

  it 'should accept connections' do
    server = Postfix::Rate::Policyd::PostfixServer.start! '127.0.0.1', 10031, pool_size: 2

    socket = nil
    expect {
      socket = TCPSocket.new '127.0.0.1', 10031   
    }.not_to raise_error
    
  ensure
    sleep 1 # allow the workers to do their magic
    socket.close   
    server.shutdown
  end

  it 'should accept multiple connections' do
    server = Postfix::Rate::Policyd::PostfixServer.start! '127.0.0.1', 10031, pool_size: 2

    sockets = []
    expect {
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sockets << TCPSocket.new('127.0.0.1', 10031)
      sleep 1
    }.not_to raise_error
    
  ensure
    sockets.each(&:close)   
    server.shutdown
  end

  it 'should process requests' do
    server = Postfix::Rate::Policyd::PostfixServer.start! '127.0.0.1', 10031, pool_size: 2

    socket = TCPSocket.new '127.0.0.1', 10031   
    expect {
      socket.sendmsg (request % ['foobar@foo.bar'])
      socket.recv 255
    }.to not_raise_error
      .and change{Postfix::Rate::Models::AuthenticatedIdentity.count}.by(1)
      .and change{Postfix::Rate::Models::Tracker.count}.by(1)
    
  ensure
    socket.close   
    server.shutdown
  end

  it 'should send a response' do
    server = Postfix::Rate::Policyd::PostfixServer.start! '127.0.0.1', 10031, pool_size: 2

    socket = TCPSocket.new '127.0.0.1', 10031   
    expect {
      socket.sendmsg (request % ['foobar@foo.bar'])
      expect(socket.recv 255).to eq "action=OK\n\n"
    }.to not_raise_error
      .and change{Postfix::Rate::Models::AuthenticatedIdentity.count}.by(1)
      .and change{Postfix::Rate::Models::Tracker.count}.by(1)
    
  ensure
    socket.close   
    server.shutdown
  end

  it 'should process multiple requests' do
    server = Postfix::Rate::Policyd::PostfixServer.start! '127.0.0.1', 10031, pool_size: 2

    sockets = [ 
      TCPSocket.new('127.0.0.1', 10031),
      TCPSocket.new('127.0.0.1', 10031),
      TCPSocket.new('127.0.0.1', 10031),
      TCPSocket.new('127.0.0.1', 10031),
      TCPSocket.new('127.0.0.1', 10031),
    ]
    expect {
      ths = []
      ths << Thread.new { 
        sockets[0].sendmsg (request % ['foobar1@foo.bar'])
        sockets[0].recv 255
        sleep 1
        sockets[0].close 
      }
      ths << Thread.new { 
        sockets[1].sendmsg (request % ['foobar2@foo.bar'])
        sockets[1].recv 255
        sleep 1
        sockets[1].close 
      }
      ths << Thread.new { 
        sockets[2].sendmsg (request % ['foobar3@foo.bar'])
        sockets[2].recv 255
        sleep 1
        sockets[2].close 
      }
      ths << Thread.new { 
        sockets[3].sendmsg (request % ['foobar4@foo.bar'])
        sockets[3].recv 255
        sleep 1
        sockets[3].close 
      }
      ths << Thread.new { 
        sockets[4].sendmsg (request % ['foobar5@foo.bar'])
        sockets[4].recv 255
        sleep 1
        sockets[4].close 
      }
      ths.each(&:join)
      sleep 1 # allow the workers to do their magic
    }.to not_raise_error
      .and change{Postfix::Rate::Models::AuthenticatedIdentity.count}.by(5)
      .and change{Postfix::Rate::Models::Tracker.count}.by(5)
  
  ensure
    server.shutdown
  end
end